// Copyright 1999, The Wilson Partnership.
// All Rights Reserved.
// @(#)SntpMessage.java, 0.1, 6 September 1999
// Author: John Wilson - tug@wilson.co.uk

package uk.co.wilson.tini.rfc2030;

import uk.co.wilson.tini.util.UDPMessage;

public final class SntpMessage extends UDPMessage.Impl {
	public SntpMessage(final SntpMessageProcessor processor) {
		super(messageLength);
		
		this.processor = processor;
	}
	
	public boolean setResponse() {
		if (packet.getLength() < 48) return false;
		
		packet.setLength(48);
		
		return processor.setResponse(this);
	}
	
	public void setLi(final int li) {
		message[0] = (byte)((message[0] & 0X3F) | (li << 6));
	}
	
	public int getVn() {
		return (message[0] >> 3) & 0X07;
	}
	
	public void setVn(final int vn) {
		message[0] = (byte)((message[0] & 0XCF) | (vn << 3));
	}
	
	public int getMode() {
		return message[0] & 0X07;
	}
	
	public void setMode(final int mode) {
		message[0] = (byte)((message[0] & 0XF8) | mode);
	}
	
	public void setStratum(final int stratum) {
		message[1] = (byte)stratum;
	}
	
	public void setPoll(final int poll) {
		message[2] = (byte)poll;
	}
	
	public void setPrecision(final int precision) {
		message[3] = (byte)precision;
	}
	
	public void setRootDelay(final byte[] rootDelay) {
		moveBytes(message, 4, rootDelay);
	}
	
	public void setRootDispersion(final byte[] rootDispersion) {
		moveBytes(message, 8, rootDispersion);
	}
	
	public void setReferenceIdentifier(final byte[] referenceIdentifier) {
		moveBytes(message, 12, referenceIdentifier);
	}
	
	public void setReferenceTimestamp(final byte[] referenceTimestamp) {
		moveBytes(message, 16, referenceTimestamp);
	}
	
	public void setOriginateTimestamp(final byte[] originateTimestamp) {
		moveBytes(message, 24, originateTimestamp);
	}
	
	public void setReceiveTimestamp(final byte[] receiveTimestamp) {
		moveBytes(message, 32, receiveTimestamp);
	}
	
	public byte[] getTransmitTimestamp() {
	final byte[] ans = new byte[8];
	
		moveBytes(ans, message, 40);
		
		return ans;
	}
	
	public void setTransmitTimestamp(final byte[] transmitTimestamp) {
		moveBytes(message, 40, transmitTimestamp);
	}
	
	private final SntpMessageProcessor processor;
	
	private static final int messageLength = 68;
}
