// Copyright 1999, The Wilson Partnership.
// All Rights Reserved.
// @(#)SntpMessageProcessor.java, 0.1, 14 September 1999
// Author: John Wilson - tug@wilson.co.uk

package uk.co.wilson.tini.rfc2030;

import uk.co.wilson.tini.util.UDPMessageProcessor;

public interface SntpMessageProcessor extends UDPMessageProcessor {
	public boolean setResponse(SntpMessage message);
	
	abstract class Impl implements SntpMessageProcessor {
		public Impl(final byte stratum,
								final byte precision,
								final byte[] rootDelay,
								final byte[] rootDispersion,
								final byte[] referenceIdentifier)
		{
			this.stratum = stratum;
			this.precision = precision;
			this.rootDelay = rootDelay;
			this.rootDispersion = rootDispersion;
			this.referenceIdentifier = referenceIdentifier;
		}
		
		public int getPort() {
			return port;
		}
		
		public boolean setResponse(final SntpMessage message) {
			if (message.getVn() == 0) return false;	// invalid version number
			
			if (message.getMode() != 3) {
				if (message.getMode() == 4) setSourceTime(message);
				
				return false;
			}
			
			message.setMode(4);
			
			if (clockSynchronized) {
			final Timestamp timestamp = getTimestamp();
			
				message.setLi(0);
				message.setStratum(stratum);
				message.setPrecision(precision);
				message.setRootDelay(rootDelay);
				message.setRootDispersion(rootDispersion);
				message.setReferenceIdentifier(referenceIdentifier);
				message.setReferenceTimestamp(timestamp.referenceTimestamp);
				message.setOriginateTimestamp(message.getTransmitTimestamp());
				message.setReceiveTimestamp(timestamp.originateTimestamp);
				message.setTransmitTimestamp(timestamp.originateTimestamp);
				
			} else {
				message.setLi(3);	// time not yet valid
				message.setStratum(0);
			}
			
			return true;
		}
		
		protected void setSourceTime(final SntpMessage message) {
			// ignore this unless handled by a subclass
		}
		
		protected static class Timestamp {
			public Timestamp(final byte[] referenceTimestamp, final byte[] originateTimestamp) {
				this.referenceTimestamp = referenceTimestamp;
				this.originateTimestamp = originateTimestamp;
			}
			
			private final byte[] referenceTimestamp;
			private final byte[] originateTimestamp;
		}

		protected abstract Timestamp getTimestamp();
		
		protected boolean clockSynchronized = false;

		private final byte stratum;
		private final byte precision;
		private final byte[] rootDelay;
		private final byte[] rootDispersion;
		private final byte[] referenceIdentifier;
		
		private static final int port = 123;
	}
}
