// Copyright 1999, The Wilson Partnership.
// All Rights Reserved.
// @(#)DayTimedImpl.java, 0.1, 13 September 1999
// Author: John Wilson - tug@wilson.co.uk

package uk.co.wilson.tini.rfc867;

import java.net.Socket;

import java.io.IOException;

import uk.co.wilson.tini.util.TCPDaemonImpl;
import uk.co.wilson.tini.util.UDPDaemonImpl;
import uk.co.wilson.tini.util.UDPMessage;
import uk.co.wilson.tini.util.Clock;

public final class DayTimedImpl {
	private DayTimedImpl() {
	}
	
	public static final class TCP extends TCPDaemonImpl {
		private TCP() {
			super(port);
		}
		
		protected void respond(final Socket socket) throws IOException {
			socket.getOutputStream().write(clock.getTime());
		}
		
		public static final Thread instance = new TCP();
	
		private final DayTimeClock clock = new DayTimeClock();
		
		private static final int port = 13;
	}

	public final static class UDP extends UDPDaemonImpl {
		private UDP() {
			super(port, new UDPMessage.Impl(0) {
												public boolean setResponse() {
													packet.setData(clock.getTime());
													packet.setLength(packet.getData().length);
														
													return true;
												}
	
												private final DayTimeClock clock = new DayTimeClock();
											});
		}
		
		public static final Thread instance = new UDP();
		
		private static final int port = 13;
	}
	
	private final static class DayTimeClock extends Clock {
		public byte[] getTime() {
			getRTC();
			
			final int fullYear = year < 70 ? year + 2000 : year + 1900;
			
			System.arraycopy(days[day - 1], 0, dateTime, 0, 4);
			System.arraycopy(months[month - 1], 0, dateTime, 4, 4);
			twoDigits(dateTime, 8, date, space);
			twoDigits(dateTime, 11, (is12Hour && pm) ? hour + 12 : hour, zero);
			twoDigits(dateTime, 14, minute, zero);
			twoDigits(dateTime, 17, second, zero);
			twoDigits(dateTime, 20, fullYear / 100, space);
			twoDigits(dateTime, 22, fullYear % 100, zero);
			
			return dateTime;
		}
	
		private static void twoDigits(final byte[] buff, final int start, final int value, final byte fill) {
		final int msb = value / 10;
		
			buff[start] = (msb == 0) ? fill : (byte)(msb + zero);
			buff[start + 1] = (byte)((value % 10) + zero);
		}
		
		private static final byte[] dateTime = new byte[24];
		
		private static final byte colon = ":".getBytes()[0];
		private static final byte space = " ".getBytes()[0];
		private static final byte zero = "0".getBytes()[0];

		private static final byte[] months[] = {"Jan ".getBytes(), "Feb ".getBytes(), "Mar ".getBytes(),
																						"Apr ".getBytes(), "May ".getBytes(), "Jun ".getBytes(),
																						"Jul ".getBytes(), "Aug ".getBytes(), "Sep ".getBytes(),
																						"Oct ".getBytes(), "Nov ".getBytes(), "Dec ".getBytes()};
		private static final byte[] days[] = {"Sun ".getBytes(), "Mon ".getBytes(), "Tue ".getBytes(),
																					"Wed ".getBytes(), "Thu ".getBytes(), "Fri ".getBytes(), "Sat ".getBytes()};
		
		static {
			dateTime[10] = space;
			dateTime[13] = colon;
			dateTime[16] = colon;
			dateTime[19] = space;
		}
	}
}